import 'package:boost/boost.dart';
import 'package:rxdart/rxdart.dart';

import 'collection_state.dart';
import 'reactive_collection.dart';

class CollectionController<Item> implements ReactiveCollection<Item> {
  final BehaviorSubject<CollectionState<Item>> _streamController =
      BehaviorSubject<CollectionState<Item>>();

  final _loadingSemaphore = Semaphore();
  final Future<Iterable<Item>> Function(int offset, int length) _query;
  final int chunkSize;
  bool _done = false;

  CollectionController(this._query, {this.chunkSize = 25}) {
    _streamController.sink.add(LoadingCollectionState<Item>(const []));
  }

  Stream<CollectionState<Item>> get stream => _streamController.stream;

  //TODO query length instead of doing last empty request
  //TODO error handling
  Future<bool> load({bool invalidate = false}) async {
    if (invalidate) {
      _streamController.sink.add(LoadingCollectionState<Item>(const []));
      _done = false;
    }

    if (!_done) {
      //TODO retrigger can be removed with count query
      bool retrigger = false;
      await _loadingSemaphore.throttle(() async {
        final current = _streamController.value;
        _streamController.sink.add(current.loading());
        final result = await _query(current.items.length, chunkSize);

        if (result.length == 0) {
          _done = true;
          _streamController.sink.add(CollectionState<Item>(current.items));
        } else {
          final resultList = <Item>[];
          for (final item in current.items) {
            resultList.add(item);
          }
          for (final item in result) {
            resultList.add(item);
          }
          _streamController.sink.add(LoadingCollectionState(resultList));

          if (result.length != chunkSize) {
            retrigger = true;
          }
        }
      });

      if (retrigger) {
        await load();
      }
    }
    return _done;
  }

  void dispose() {
    _streamController.close();
  }
}
