import 'package:cl_appbase/appbase.dart';

/// State for paged, loading collections.
///
/// List with or without items -> [CollectionState]
/// Loading more items -> [LoadingCollectionState]
/// An error occurred -> [ErrorCollectionState]
///
/// See [CollectionBuilder] for more.
class CollectionState<T> {
  final List<T> items;

  CollectionState(this.items);

  /// Returns a CollectionState with the items of this state and [addedItems].
  CollectionState<T> add(Iterable<T> addedItems) {
    return CollectionState(
        items.followedBy(addedItems).toList(growable: false));
  }

  /// Returns a LoadingCollectionState with the same items as this state.
  CollectionState<T> loading() {
    return LoadingCollectionState(items);
  }
}

class LoadingCollectionState<T> extends CollectionState<T> {
  LoadingCollectionState(List<T> items) : super(items);
}

class ErrorCollectionState<T> extends CollectionState<T> {
  final dynamic error;

  ErrorCollectionState({this.error}) : super([]);
}
