import 'package:cl_appbase/src/bloc/collection_state.dart';
import 'package:flutter/material.dart';

/// Provides an easy wrapper for a builder presenting the value of
/// a [CollectionState] stream.
class CollectionBuilder<TValue> extends StatelessWidget {
  final Stream<CollectionState<TValue>> stream;
  final Widget Function(BuildContext context, List<TValue> items, bool loading)
      builder;
  final Widget Function(BuildContext context, dynamic error) error;
  final Widget Function(BuildContext context)? empty;

  const CollectionBuilder({
    Key? key,
    required this.stream,
    required this.builder,
    this.empty,
    required this.error,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<CollectionState<TValue>>(
        stream: stream, builder: _builder);
  }

  Widget _builder(
      BuildContext context, AsyncSnapshot<CollectionState<TValue>> snapshot) {
    if (snapshot.hasError) {
      return error(context, snapshot.error);
    } else if (snapshot.hasData) {
      final state = snapshot.requireData;
      if (state is ErrorCollectionState<TValue>) {
        return error(context, state.error);
      } else {
        if (state.items.isEmpty &&
            state is! LoadingCollectionState &&
            empty != null) {
          return empty!(context);
        } else {
          return builder(context, state.items, state is LoadingCollectionState);
        }
      }
    } else {
      return error(context, null);
    }
  }
}
