import 'package:flutter/material.dart';

import '../property_state.dart';

/// Provides an easy wrapper for a builder presenting the value of
/// a [PropertyState] stream.
class PropertyBuilder<TValue> extends StatelessWidget {
  final Stream<PropertyState<TValue>> stream;
  final Widget Function(BuildContext context, TValue state) value;
  final Widget Function(BuildContext context) loading;
  final Widget Function(BuildContext context, dynamic error) error;

  const PropertyBuilder({
    Key? key,
    required this.stream,
    required this.value,
    required this.loading,
    required this.error,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<PropertyState<TValue>>(
        stream: stream.distinct(), builder: _builder);
  }

  Widget _builder(
      BuildContext context, AsyncSnapshot<PropertyState<TValue>> snapshot) {
    if (snapshot.hasError) {
      return error(context, snapshot.error);
    } else if (snapshot.hasData) {
      final state = snapshot.requireData;
      if (state is ValueState<TValue>) {
        return value(context, state.value);
      } else if (state is LoadingState) {
        return loading(context);
      } else if (state is UnavailableState<TValue>) {
        return error(context, state.error);
      } else {
        return error(context, 'Invalid state: ${state.runtimeType}');
      }
    } else {
      return error(context, null);
    }
  }
}
